import datetime

import pytest
from sqlalchemy import select, text, create_engine
from sqlalchemy.orm import Session

from oldw.schema import SourceSystem

from .settings import ORACLE_CONNECTION_STRING


@pytest.fixture(scope="module")
def db_session():
    engine = create_engine(ORACLE_CONNECTION_STRING)
    with Session(engine) as session:
        yield session


def test_connect_oracle(db_session):
    query = text("SELECT 1 AS x FROM DUAL")
    cursor_result = db_session.execute(query)
    result = cursor_result.fetchone()
    assert len(result) == 1
    assert result[0] == 1


def test_source_system(db_session):
    query = select(SourceSystem)
    results = db_session.execute(query).all()[0]
    assert len(results) == 1
    first_source_system = results[0]
    assert first_source_system.id == "TEST"
    assert len(first_source_system.courses) == 1


def test_all_models_queryable(db_session):
    from oldw import schema

    schema_imports = [getattr(schema, name) for name in dir(schema)]
    models = [
        m
        for m in schema_imports
        if isinstance(m, type) and issubclass(m, schema.Base) and m is not schema.Base
    ]

    for model in models:
        db_session.query(model).count()


def test_that_timestamps_are_timezone_aware(db_session):
    from oldw.schema import Course

    results = db_session.execute(select(Course)).all()[0]
    first_course: Course = results[0]
    launched_timestamp = first_course.launched
    assert launched_timestamp is not None
    assert launched_timestamp.utcoffset() is not None
    assert launched_timestamp.utcoffset().total_seconds() == 0
    assert launched_timestamp.date() == datetime.date(2020, 1, 1)
    assert launched_timestamp.time() == datetime.time()
